'''
Created on 2023-08-16

@author: wf
'''
from nicesrt.srt import SRT
from tests.basetest import Basetest

class Test_SRT(Basetest):

    def test_srt(self):
        srt_text="""1
00:00:00,000 --> 00:00:00,033
<font size="28">SrtCnt : 1, DiffTime : 33ms
2023-08-15 09:18:24.589
[iso : 200] [shutter : 1/180.0] [fnum : 170] [ev : 1.3] [ct : 5490] [color_md : default] [focal_len : 240] [dzoom_ratio: 10000, delta:0],[latitude: 48.486375] [longitude: 8.375567] [rel_alt: 0.000 abs_alt: 530.095] </font>

2
00:00:00,033 --> 00:00:00,066
<font size="28">SrtCnt : 2, DiffTime : 33ms
2023-08-15 09:18:24.622
[iso : 200] [shutter : 1/180.0] [fnum : 170] [ev : 1.3] [ct : 5490] [color_md : default] [focal_len : 240] [dzoom_ratio: 10000, delta:0],[latitude: 48.486375] [longitude: 8.375566] [rel_alt: 0.000 abs_alt: 530.095] </font>

3
00:00:00,066 --> 00:00:00,100
<font size="28">SrtCnt : 3, DiffTime : 34ms
2023-08-15 09:18:24.655
[iso : 200] [shutter : 1/180.0] [fnum : 170] [ev : 1.3] [ct : 5490] [color_md : default] [focal_len : 240] [dzoom_ratio: 10000, delta:0],[latitude: 48.486375] [longitude: 8.375566] [rel_alt: 0.000 abs_alt: 530.095] </font>
"""
        # Load the SRT text using the from_text classmethod
        srt = SRT.from_text(srt_text)
    
        # Extract the latitude and longitude from the first subtitle
        lat1 = float(srt.extract_value(0, "latitude"))
        lon1 = float(srt.extract_value(0, "longitude"))
    
        self.assertAlmostEqual(lat1, 48.486375, delta=0.000001)
        self.assertAlmostEqual(lon1, 8.375567, delta=0.000001)


"""
Created on 2023-06-19

@author: wf
"""
from typing import Optional
from nicesrt.version import Version
from nicesrt.local_filepicker import LocalFilePicker
from nicesrt.file_selector import FileSelector
from nicegui import ui

import os
import sys
import requests
import traceback


class WebServer:
    """
    WebServer class that manages the server 
    
    """

    def __init__(self):
        """Constructs all the necessary attributes for the WebServer object."""
        self.is_local=False
        self.log_view=None
        self.do_trace=True
        self.input=""
        
 
        @ui.page('/')
        async def home():
            await self.home()
            
        @ui.page('/settings')
        def settings():
            self.settings()
            
            
    @classmethod
    def examples_path(cls)->str:
        # the root directory (default: examples)
        path = os.path.join(os.path.dirname(__file__), '../nicesrt_examples')
        path = os.path.abspath(path)
        return path
 
    def handle_exception(self, e: BaseException, trace: Optional[bool] = False):
        """Handles an exception by creating an error message.

        Args:
            e (BaseException): The exception to handle.
            trace (bool, optional): Whether to include the traceback in the error message. Default is False.
        """
        if trace:
            self.error_msg = str(e) + "\n" + traceback.format_exc()
        else:
            self.error_msg = str(e)
        if self.log_view:
            self.log_view.push(self.error_msg)
        print(self.error_msg,file=sys.stderr)

    async def render(self, _click_args=None):
        """
        Renders the SRT content

        Args:
            click_args (object): The click event arguments.
        """
        try:
            ui.notify("rendering ...")
            # show render result in log
            #self.log_view.push(render_result.stderr)
        except BaseException as ex:
            self.handle_exception(ex,self.do_trace)  
            
    def do_read_input(self, input_str: str):
        """Reads the given input.

        Args:
            input_str (str): The input string representing a URL or local path.
        """
        if input_str.startswith('http://') or input_str.startswith('https://'):
            response = requests.get(input_str)
            if response.status_code == 200:
                return response.text
            else:
                raise Exception(f'Unable to retrieve data from URL: {input_str}')
        else:
            if os.path.exists(input_str):
                with open(input_str, 'r') as file:
                    return file.read()
            else:
                raise Exception(f'File does not exist: {input_str}')
    
    async def read_and_optionally_render(self,input_str):
        """Reads the given input and optionally renders the given input

        Args:
            input_str (str): The input string representing a URL or local file.
        """
        self.read_input(input_str)
        if self.render_on_load:
            await self.render(None)
        
    def read_input(self, input_str: str):
        """Reads the given input and handles any exceptions.

        Args:
            input_str (str): The input string representing a URL or local file.
        """
        try:
            ui.notify(f"reading {input_str}")
            self.log_view.clear()
            self.error_msg = None
        except BaseException as e:
            self.handle_exception(e, self.do_trace)
            
    async def open_file(self) -> None:
        """Opens a Local filer picker dialog and reads the selected input file."""
        if self.is_local:
            pick_list = await LocalFilePicker('~', multiple=False)
            if len(pick_list)>0:
                input_file=pick_list[0]
                await self.read_and_optionally_render(input_file)
    pass
         
    
    def link_button(self, name: str, target: str, icon_name: str):
        """
        Creates a button with a specified icon that opens a target URL upon being clicked.
    
        Args:
            name (str): The name to be displayed on the button.
            target (str): The target URL that should be opened when the button is clicked.
            icon_name (str): The name of the icon to be displayed on the button.
    
        Returns:
            The button object.
        """
        with ui.button(name,icon=icon_name) as button:
            button.on("click",lambda: (ui.open(target)))
        return button
    
    
    def tool_button(self,tooltip:str,icon:str,handler:callable=None,toggle_icon:str=None)->ui.button:
        """
        Creates an  button with icon that triggers a specified function upon being clicked.
    
        Args:
            tooltip (str): The tooltip to be displayed.
            icon (str): The name of the icon to be displayed on the button.
            handler (function): The function to be called when the button is clicked.
            toggle_icon (str): The name of an alternative icon to be displayed when the button is clicked.
    
        Returns:
            ui.button: The icon button object.
            
        valid icons may be found at:    
            https://fonts.google.com/icons
        """
        icon_button=ui.button("",icon=icon, color='primary').tooltip(tooltip).on("click",handler=handler)  
        icon_button.toggle_icon=toggle_icon
        return icon_button   
        
    def setup_menu(self):
        """Adds a link to the project's GitHub page in the web server's menu."""
        with ui.header() as self.header:
            self.link_button("home","/","home")
            self.link_button("settings","/settings","settings")
            self.link_button("github",Version.cm_url,"bug_report")
            self.link_button("chat",Version.chat_url,"chat")
            self.link_button("help",Version.doc_url,"help")
    
    def setup_footer(self):
        """
        setup the footer
        """
        with ui.footer() as self.footer:
            ui.label("(c)2023 Wolfgang Fahl")
            ui.link("Powered by nicegui","https://nicegui.io/").style("color: #fff") 
  
    def input_changed(self,cargs):
        """
        react on changed input
        """
        self.input=cargs.value
        pass
    
    def code_changed(self,cargs):
        """
        react on changed code
        """
        self.code=cargs.value
        pass
    
    def toggle_icon(self,button:ui.button):
        """
        toggle the icon of the given button
        
        Args:
            ui.button: the button that needs the icon to be toggled
        """
        if hasattr(button,"toggle_icon"):
            # exchange icon with toggle icon
            toggle_icon=button._props["icon"]
            icon=button.toggle_icon
            button._props["icon"]=icon
            button.toggle_icon=toggle_icon
        button.update()
        
    async def home(self):
        """Generates the home page with a map"""
        self.setup_menu()
        with ui.column():
            with ui.splitter() as splitter:
                with splitter.before:        
                    with ui.scene(width=1024, height=768).classes("w-full") as scene:
                        self.scene = scene
                        scene.spot_light(distance=100, intensity=0.2).move(-10, 0, 10)
                    with splitter.after:
                        with ui.element("div").classes("w-full"):
                            self.example_selector=FileSelector(path=self.root_path,handler=self.read_and_optionally_render)
                            self.input_input=ui.input(
                                value=self.input,
                                on_change=self.input_changed).props("size=100")                     
        self.setup_footer()        
        if self.args.input:
            await self.read_and_optionally_render(self.args.input)
        
    def settings(self):
        """Generates the settings page with a link to the project's GitHub page."""
        self.setup_menu()
        ui.checkbox('debug with trace', value=True).bind_value(self, "do_trace")
        ui.checkbox('render on load',value=self.render_on_load).bind_value(self,"render_on_load")
        self.setup_footer()
       
    def run(self, args):
        """Runs the UI of the web server.

        Args:
            args (list): The command line arguments.
        """
        self.args=args
        self.is_local=args.local
        self.root_path=os.path.abspath(args.root_path) 
        self.render_on_load=args.render_on_load
        ui.run(title=Version.name, host=args.host, port=args.port, show=args.client,reload=False)



'''
Created on 2023-07-24

@author: wf
'''
from typing import Callable, Dict, List, Optional, Union
from nicegui import ui
from nicegui.events import ValueChangeEventArguments
import inspect
import os

class FileSelector():
    """
    nicegui FileSelector
    """
    def __init__(self,path:str,extensions: dict=None,handler:Callable=None):
        """
        constructor
        
        Args:
            path (str): The path to the directory to start building the tree from.
            extensions(dict): the extensions to filter for as a dictionary with name as key and extension as value
            handler(Callable): handler function to call on selection
        """   
        self.path=path
        if extensions is None:
            extensions = {"srt": ".SRT"}
        self.extensions=extensions 
        self.handler=handler
        # generate the tree structure
        self.tree_structure = self.get_dir_tree(self.path, self.extensions)

        # create the ui.tree object
        self.tree=ui.tree([self.tree_structure], label_key='label', on_select=self.select_file)
   
    def get_dir_tree(self, path: str, extensions: dict, id_path: List[int] = [1]) -> Optional[Dict[str, Union[str, List[Dict]]]]:
        """
        Recursive function to construct a directory tree.
    
        Args:
            path (str): The path to the directory to start building the tree from.
            extensions(dict): the extensions to filter for as a dictionary with name as key and extension as value
            id_path (List[int]): List of integers representing the current path in the tree.
    
        Returns:
            dict: A dictionary representing the directory tree. For each directory or .scad file found,
            it will add a dictionary to the 'children' list of its parent directory's dictionary.
        """
        path = os.path.abspath(path)
        id_string = '.'.join(map(str, id_path))
        items = os.listdir(path)
        children = []
        item_counter = 1  # counter for generating child id
    
        # Iterating over directories first
        for name in items:
            item_path = os.path.join(path, name)
            child_id_path = id_path + [item_counter]
    
            if os.path.isdir(item_path):
                dir_tree = self.get_dir_tree(item_path, extensions, child_id_path)
                if dir_tree:
                    children.append(dir_tree)
                    item_counter += 1
    
        # Then iterating over files
        for name in items:
            item_path = os.path.join(path, name)
            child_id_path = id_path + [item_counter]
    
            if name.endswith(tuple(extensions.values())) and not os.path.isdir(item_path):
                children.append({
                    'id': '.'.join(map(str, child_id_path)),
                    'label': name,
                    'value': item_path,
                })
                item_counter += 1
    
        if children:
            return {
                'id': id_string,
                'label': os.path.basename(path),
                'value': path,
                'children': children,
            }



    def find_node_by_id(self, tree: Dict[str, Union[str, List[Dict]]], id_to_find: str) -> Optional[Dict[str, Union[str, List[Dict]]]]:
        """
        Recursive function to find a node (file or directory) by its ID in a directory tree.
    
        Args:
            tree (dict): A dictionary representing the directory tree. The tree is constructed with each node 
                containing 'id' (str) as a unique identifier, 'value' (str) as the path to the file or directory,
                and 'children' (list of dict) as a list of child nodes.
            id_to_find (str): The ID of the node to find in the directory tree.
    
        Returns:
            dict: The node associated with the found ID. Returns None if the ID is not found.
        """
        if tree['id'] == id_to_find:
            return tree
        
        for child in tree.get('children', []):
            found = self.find_node_by_id(child, id_to_find)
            if found:
                return found
                
        return None
    
    def expand(self,node_ids: List[str]) -> None:
        """
        expand the nodes with the given ids
        
        node_ids(list): the list of node ids to be expanded
        """
        self.tree._props['expanded'] = node_ids
        self.tree.update()
    
    async def select_file(self,vcea:ValueChangeEventArguments):
        """
        select the given file and call my handler on the file path of it
        
        Args:
            vcea(ValueChangeEventArguments): the tree selection event
        """
        id_to_find = vcea.value  # Assuming vcea.value contains the id
        if id_to_find is None:
            return 
        
        selected_node = self.find_node_by_id(self.tree_structure, id_to_find)
        if selected_node is None:
            raise ValueError(f"No item with id {id_to_find} found in the tree structure.")
        
        file_path = selected_node['value']
        
        # Only expand the node if it is a directory
        if os.path.isdir(file_path):
            # Access the children of the selected node and get their ids
            if 'children' in selected_node:
                child_ids = [child['id'] for child in selected_node['children']]
                self.expand(child_ids)
        else:
            if self.handler:
                if inspect.iscoroutinefunction(self.handler):
                    await self.handler(file_path)
                else:
                    self.handler(file_path) 
                # Close all nodes if it is a file
            self.expand([])
   






'''
https://github.com/zauberzeug/nicegui/blob/main/examples/map/leaflet.py

@author: rodja
'''
from typing import Tuple

from nicegui import ui


class leaflet(ui.element, component='leaflet.js'):

    def __init__(self) -> None:
        super().__init__()
        ui.add_head_html('<link href="https://unpkg.com/leaflet@1.6.0/dist/leaflet.css" rel="stylesheet"/>')
        ui.add_head_html('<script src="https://unpkg.com/leaflet@1.6.0/dist/leaflet.js"></script>')

    def set_location(self, location: Tuple[float, float]) -> None:
        self.run_method('set_location', location[0], location[1])


'''
Created on 2023-08-16

@author: wf
'''
import pysrt


class SRT:
    """
    Class to represent and manipulate SRT (SubRip Subtitle) data.
    
    Attributes:
        subtitles (list): List of parsed subtitles.
    """

    def __init__(self, subtitles):
        """
        Initializes the SRT object with the given subtitles.

        Args:
            subtitles (list): List of parsed subtitles.
        """
        self.subtitles = subtitles

    @classmethod
    def from_text(cls, text):
        """
        Class method to create an SRT object from a raw text string.

        Args:
            text (str): Raw SRT formatted string.

        Returns:
            SRT: An instance of the SRT class initialized with parsed subtitles.
        """
        subtitles = pysrt.from_string(text)
        return cls(subtitles)

    def extract_value(self, index, key):
        """
        Extracts the value for a given key from the subtitle at the specified index.

        Args:
            index (int): The index of the subtitle from which the value should be extracted.
            key (str): The key for which the value should be extracted.

        Returns:
            str: Extracted value for the given key, or None if key is not found.
        """
        text = self.subtitles[index].text
        try:
            start_index = text.index(f"[{key}:") + len(key) + 2
            end_index = text.index("]", start_index)
            return text[start_index:end_index].strip()
        except ValueError:
            return None
